#!/usr/bin/env python3
import sys, os, base64, datetime, hashlib, hmac 
import requests # pip install requests
import traceback
import time
import common
from urllib import urlencode

if(hasattr(requests.packages.urllib3, 'disable_warnings')):
    requests.packages.urllib3.disable_warnings()

def sendMsg(method, service, host, region, canonical_uri, access_key, secret_key, secret_token, request_parameters, time_out):
    logger = common.getLogger()
    if access_key is None or secret_key is None:
        logger.error('No access key is available.')
        raise Exception("No access key is available.")
    
    content_type = 'application/json'

    endpoint = 'https://' + host + canonical_uri[:-1]
    # Create a date for headers and the credential string
    time_now = datetime.datetime.utcnow()
    amz_date = time_now.strftime('%Y%m%dT%H%M%SZ')
    date_stamp = time_now.strftime('%Y%m%d') # Date w/o time, used in credential scope
    if('GET' == method):
        logger.debug("request_parameters is %s"%(request_parameters))# canonical_querystring = request_parameters
        paramdict = parseParams(request_parameters)
        canonical_querystring = urlencode(sorted(paramdict.items(), key=lambda d:d[0]))
        logger.debug("canonical_querystring is %s"%(canonical_querystring))
    elif('POST' == method):
        canonical_querystring = ''
    else:
        raise Exception("Does't support method %s."%(method))
    canonical_headers = 'host:' + host + '\n' + 'x-sdk-date:' + amz_date + '\n'

    signed_headers = 'host;x-sdk-date'

    # Create payload hash. In this example, the payload (body of
    # the request) contains the request parameters.
    if('POST' == method):
        payload_hash = hashlib.sha256(request_parameters.encode('utf-8')).hexdigest()
    else:
        payload_hash = hashlib.sha256(''.encode('utf-8')).hexdigest()

    # Combine elements to create create canonical request
    canonical_request = method + '\n' + canonical_uri + '\n' + canonical_querystring + '\n' + canonical_headers + '\n' + signed_headers + '\n' + payload_hash
    logger.debug('canonical_request:' + canonical_request)

    # Match the algorithm to the hashing algorithm you use, either SHA-1 or
    # SHA-256 (recommended)
    algorithm = 'SDK-HMAC-SHA256'
    credential_scope = date_stamp + '/' + region + '/' + service + '/' + 'sdk_request'
    string_to_sign = algorithm + '\n' +  amz_date + '\n' +  credential_scope + '\n' +  hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()
    logger.debug('string_to_sign:' + string_to_sign)

    # Create the signing key using the function defined above.
    signing_key = getSignatureKey(secret_key, date_stamp, region, service)

    # Sign the string_to_sign using the signing_key
    signature = hmac.new(signing_key, (string_to_sign).encode('utf-8'), hashlib.sha256).hexdigest()

    # Put the signature information in a header named Authorization.
    authorization_header = algorithm + ' ' + 'Credential=' + access_key + '/' + credential_scope + ', ' +  'SignedHeaders=' + signed_headers + ', ' + 'Signature=' + signature

    headers = {'Content-Type':content_type, 
               'Host':host,
               'X-Sdk-Date':amz_date, 
               'Authorization':authorization_header,
               'X-Security-Token':secret_token}

    # ************* SEND THE REQUEST *************
    request_url = endpoint

    # ************* SEND THE REQUEST *************
    logger.info('Request URL = ' + request_url)
    response=''
    if('GET' == method):
        response = requests.get(request_url, headers=headers, verify=False, params=parseParams(request_parameters), timeout=time_out)
    else:
        response = requests.post(endpoint, data=request_parameters, headers=headers, verify=False, timeout=time_out)
    logger.info('Response code: %d' % response.status_code)
    logger.debug(response.text)
    return response.status_code,response.text


def sign(key, msg):
    return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

def getSignatureKey(key, dateStamp, regionName, serviceName):
    kDate = sign(('SDK' + key).encode('utf-8'), dateStamp)
    kRegion = sign(kDate, regionName)
    kService = sign(kRegion, serviceName)
    kSigning = sign(kService, 'sdk_request')
    return kSigning

def parseParams(request_parameters):
    paramDict = {}
    if(request_parameters):
        params = request_parameters.split('&')
        for param in params:
            pair = param.split('=')
            paramDict[pair[0]] = pair[1]
    return paramDict

def registVm(host, tenant_id, server_id, access_key, secret_key, secret_token, time_out):
    method = 'POST'
    service = 'ec2'
    canonical_uri = '/v1.0/servers/%s/action/'%(server_id)
    region = 'eu-de'
    request_parameters =  '{'
    request_parameters +=  '"monitorMetrics":"None"'
    request_parameters +=  '}'
    return sendMsg(method, service, host, region, canonical_uri, access_key, secret_key, secret_token, request_parameters, time_out)


def getMetricsFromCes(host, tenant_id, server_id, access_key, secret_key, secret_token, time_out):
    method = 'GET'
    service = 'ec2'
    canonical_uri = '/V1.0/%s/event-data/'%(tenant_id)
    region = 'eu-de'
    time_now = int(time.time()*1000)
    time_to = time_now + 120*1000
    time_from = time_now - 120*1000
    request_parameters = 'namespace=SYS.ECS&dim.0=instance_id,%s&type=instance_host_info&from=%s&to=%s'%(server_id,time_from,time_to)
    return sendMsg(method, service, host, region, canonical_uri, access_key, secret_key, secret_token, request_parameters, time_out)

